import {
  scaleLinear,
  scaleTime,
  max,
  timeFormat,
  extent,
  histogram as bin,
  timeMonths,
  mean,
  brushX,
  select,
  event,
} from 'd3';
import { useRef, useEffect, useMemo } from 'react';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const margin = {
  top: 5,
  right: 15,
  bottom: 15,
  left: 40,
};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 20;

export const DateHistogram = ({
  data,
  height,
  width,
  setBrushExtent,
  xValue,
  yValue,
}) => {
  const brushRef = useRef();

  const xAxisLabel = 'Time';
  const yAxisLabel = 'AQI Value';

  const innerHeight =
    height - margin.top - margin.bottom;
  const innerWidth =
    width - margin.left - margin.right;

  const xAxisTickFormat = timeFormat('%b');

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(data, xValue))
        .range([0, innerWidth])
        .nice(),
    [data, xValue, innerWidth]
  );

  const binnedData = useMemo(() => {
    const [start, stop] = xScale.domain();
    return bin()
    .value(xValue)
    .domain(xScale.domain())
    .thresholds(timeMonths(start, stop))(data)
    .map((array) => ({
      y: mean(array, yValue),
      x0: array.x0,
      x1: array.x1,
    }));
  }, [xValue, xScale, data, yValue]);

  const yScale = useMemo(() => scaleLinear()
    .domain([0, max(binnedData, (d) => d.y)])
    .range([innerHeight, 0]), [binnedData, innerHeight]);

  useEffect(() => {
    const brush = brushX().extent([
      [0, 0],
      [innerWidth, innerHeight],
    ]);
    brush(select(brushRef.current));
    brush.on('brush end', () => {
      setBrushExtent(
        event.selection &&
          event.selection.map(xScale.invert)
      );
    });
  }, [innerWidth, innerHeight]);

  return (
    <>
      <rect
        width={width}
        height={height}
        fill="white"
      />
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={5}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${
            innerHeight / 2
          }) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
        <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={7} />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <Marks
          binnedData={binnedData}
          xScale={xScale}
          yScale={yScale}
          tooltipFormat={(d) => d}
          circleRadius={2}
          innerHeight={innerHeight}
        />
        <g ref={brushRef} />
      </g>
    </>
  );
};
